@@ -3,16 +3,20 @@ from enum import Enum
 from random import randint
 from typing import List, Dict
 
+from flatbuffers import Builder
 from rlbot.matchconfig.loadout_config import LoadoutConfig
+from rlbot.messages.flat.PlayerClass import PlayerClass
 from rlbot.parsing.bot_config_bundle import get_bot_config_bundle
 from rlbot.parsing.agent_config_parser import load_bot_appearance
 from rlbot.parsing.match_settings_config_parser import boost_amount_mutator_types, map_types, game_mode_types, \
     match_length_types, max_score_types, overtime_mutator_types, series_length_mutator_types, game_speed_mutator_types, \
     ball_max_speed_mutator_types, ball_type_mutator_types, ball_weight_mutator_types, ball_size_mutator_types, \
     ball_bounciness_mutator_types, rumble_mutator_types, boost_strength_mutator_types, gravity_mutator_types, \
-    demolish_mutator_types, respawn_time_mutator_types, existing_match_behavior_types
+    demolish_mutator_types, respawn_time_mutator_types, existing_match_behavior_types, game_map_dict
+from rlbot.utils.logging_utils import get_logger
 from rlbot.utils.structures.start_match_structures import MatchSettings, PlayerConfiguration, MutatorSettings
-from rlbot.messages.flat import MatchSettings as MatchSettingsFlat, MutatorSettings as MutatorSettingsFlat
+from rlbot.messages.flat import MatchSettings as MatchSettingsFlat, MutatorSettings as MutatorSettingsFlat, RLBotPlayer, \
+    PsyonixBotPlayer, HumanPlayer, PlayerConfiguration as PlayerConfigurationFlat
 
 # We pass messages in flatbuffer format to RLBot.exe. In flatbuffer, a signed int field
 # is 32 bit, so it has a max value of 2^31 - 1, in other words 2147483647.
@@ -72,6 +76,35 @@ class PlayerConfig:
         if self.loadout_config:
             self.loadout_config.write(player_configuration)
 
+    def write_to_flatbuffer(self, builder: Builder, name_dict: dict):
+        name = builder.CreateString(get_sanitized_bot_name(name_dict, self.name))
+
+        if self.bot:
+            loadout = self.loadout_config.write_to_flatbuffer(builder)
+            if self.rlbot_controlled:
+                variety = PlayerClass.RLBotPlayer
+                RLBotPlayer.RLBotPlayerStart(builder)
+                player = RLBotPlayer.RLBotPlayerEnd(builder)
+            else:
+                variety = PlayerClass.PsyonixBotPlayer
+                PsyonixBotPlayer.PsyonixBotPlayerStart(builder)
+                PsyonixBotPlayer.PsyonixBotPlayerAddBotSkill(builder, self.bot_skill)
+                player = PsyonixBotPlayer.PsyonixBotPlayerEnd(builder)
+        else:
+            loadout = LoadoutConfig().write_to_flatbuffer(builder)
+            variety = PlayerClass.HumanPlayer
+            HumanPlayer.HumanPlayerStart(builder)
+            player = HumanPlayer.HumanPlayerEnd(builder)
+
+        PlayerConfigurationFlat.PlayerConfigurationStart(builder)
+        PlayerConfigurationFlat.PlayerConfigurationAddName(builder, name)
+        PlayerConfigurationFlat.PlayerConfigurationAddLoadout(builder, loadout)
+        PlayerConfigurationFlat.PlayerConfigurationAddTeam(builder, self.team)
+        PlayerConfigurationFlat.PlayerConfigurationAddVariety(builder, player)
+        PlayerConfigurationFlat.PlayerConfigurationAddVarietyType(builder, variety)
+        PlayerConfigurationFlat.PlayerConfigurationAddSpawnId(builder, self.spawn_id)
+        return PlayerConfigurationFlat.PlayerConfigurationEnd(builder)
+
     def has_bot_script(self) -> bool:
         return self.rlbot_controlled
 
@@ -144,6 +177,26 @@ class MutatorConfig:
         mutator_settings.demolish_option = index_or_zero(demolish_mutator_types, self.demolish)
         mutator_settings.respawn_time_option = index_or_zero(respawn_time_mutator_types, self.respawn_time)
 
+    def write_to_flatbuffer(self, builder: Builder):
+        MutatorSettingsFlat.MutatorSettingsStart(builder)
+        MutatorSettingsFlat.MutatorSettingsAddMatchLength(builder, index_or_zero(match_length_types, self.match_length))
+        MutatorSettingsFlat.MutatorSettingsAddMaxScore(builder, index_or_zero(max_score_types, self.max_score))
+        MutatorSettingsFlat.MutatorSettingsAddOvertimeOption(builder, index_or_zero(overtime_mutator_types, self.overtime))
+        MutatorSettingsFlat.MutatorSettingsAddSeriesLengthOption(builder, index_or_zero(series_length_mutator_types, self.series_length))
+        MutatorSettingsFlat.MutatorSettingsAddGameSpeedOption(builder, index_or_zero(game_speed_mutator_types, self.game_speed))
+        MutatorSettingsFlat.MutatorSettingsAddBallMaxSpeedOption(builder, index_or_zero(ball_max_speed_mutator_types, self.ball_max_speed))
+        MutatorSettingsFlat.MutatorSettingsAddBallTypeOption(builder, index_or_zero(ball_type_mutator_types, self.ball_type))
+        MutatorSettingsFlat.MutatorSettingsAddBallWeightOption(builder, index_or_zero(ball_weight_mutator_types, self.ball_weight))
+        MutatorSettingsFlat.MutatorSettingsAddBallSizeOption(builder, index_or_zero(ball_size_mutator_types, self.ball_size))
+        MutatorSettingsFlat.MutatorSettingsAddBallBouncinessOption(builder, index_or_zero(ball_bounciness_mutator_types, self.ball_bounciness))
+        MutatorSettingsFlat.MutatorSettingsAddBoostOption(builder, index_or_zero(boost_amount_mutator_types, self.boost_amount))
+        MutatorSettingsFlat.MutatorSettingsAddRumbleOption(builder, index_or_zero(rumble_mutator_types, self.rumble))
+        MutatorSettingsFlat.MutatorSettingsAddBoostStrengthOption(builder, index_or_zero(boost_strength_mutator_types, self.boost_strength))
+        MutatorSettingsFlat.MutatorSettingsAddGravityOption(builder, index_or_zero(gravity_mutator_types, self.gravity))
+        MutatorSettingsFlat.MutatorSettingsAddDemolishOption(builder, index_or_zero(demolish_mutator_types, self.demolish))
+        MutatorSettingsFlat.MutatorSettingsAddRespawnTimeOption(builder, index_or_zero(respawn_time_mutator_types, self.respawn_time))
+        return MutatorSettingsFlat.MutatorSettingsEnd(builder)
+
     def __eq__(self, other):
         return isinstance(other, self.__class__) and self.__dict__ == other.__dict__
 
@@ -186,18 +239,19 @@ class MatchConfig:
         self.player_configs: List[PlayerConfig] = []
         self.game_mode: str = None
         self.game_map: str = None
-        self.skip_replays: bool = None
-        self.instant_start: bool = None
+        self.skip_replays: bool = False
+        self.instant_start: bool = False
         self.mutators: MutatorConfig = None
         self.extension_config: ExtensionConfig = None
         self.existing_match_behavior: str = None
-        self.enable_lockstep: bool = None
+        self.enable_lockstep: bool = False
         self.networking_role: str = None
         self.network_address: str = None
-        self.enable_rendering: bool = None
-        self.enable_state_setting: bool = None
-        self.auto_save_replay: bool = None
+        self.enable_rendering: bool = False
+        self.enable_state_setting: bool = False
+        self.auto_save_replay: bool = False
         self.script_configs: List[ScriptConfig] = []
+        self.logger = get_logger('match_config')
 
     @property
     def num_players(self):
@@ -224,6 +278,42 @@ class MatchConfig:
 
         return match_settings
 
+    def create_flatbuffer(self) -> Builder:
+        builder = Builder(1000)
+        name_dict = {}
+        player_config_offsets = [pc.write_to_flatbuffer(builder, name_dict) for pc in self.player_configs]
+        MatchSettingsFlat.MatchSettingsStartPlayerConfigurationsVector(builder, len(player_config_offsets))
+        for i in reversed(range(0, len(player_config_offsets))):
+            builder.PrependUOffsetTRelative(player_config_offsets[i])
+        player_list_offset = builder.EndVector(len(player_config_offsets))
+        mutator_settings_offset = self.mutators.write_to_flatbuffer(builder)
+
+        if self.game_map in game_map_dict:
+            upk = game_map_dict[self.game_map]
+            map_index = map_types.index(self.game_map)
+        else:
+            self.logger.info(f"Did not recognize {self.game_map}, hoping it's new or a custom map!")
+            upk = self.game_map
+            map_index = -1
+        upk_offset = builder.CreateString(upk)
+
+        MatchSettingsFlat.MatchSettingsStart(builder)
+        MatchSettingsFlat.MatchSettingsAddPlayerConfigurations(builder, player_list_offset)
+        MatchSettingsFlat.MatchSettingsAddGameMode(builder, game_mode_types.index(self.game_mode))
+        MatchSettingsFlat.MatchSettingsAddGameMap(builder, map_index)
+        MatchSettingsFlat.MatchSettingsAddGameMapUpk(builder, upk_offset)
+        MatchSettingsFlat.MatchSettingsAddSkipReplays(builder, self.skip_replays)
+        MatchSettingsFlat.MatchSettingsAddInstantStart(builder, self.instant_start)
+        MatchSettingsFlat.MatchSettingsAddMutatorSettings(builder, mutator_settings_offset)
+        MatchSettingsFlat.MatchSettingsAddExistingMatchBehavior(builder, index_or_zero(existing_match_behavior_types, self.existing_match_behavior))
+        MatchSettingsFlat.MatchSettingsAddEnableLockstep(builder, self.enable_lockstep)
+        MatchSettingsFlat.MatchSettingsAddEnableRendering(builder, self.enable_rendering)
+        MatchSettingsFlat.MatchSettingsAddEnableStateSetting(builder, self.enable_state_setting)
+        MatchSettingsFlat.MatchSettingsAddAutoSaveReplay(builder, self.auto_save_replay)
+        ms_offset = MatchSettingsFlat.MatchSettingsEnd(builder)
+        builder.Finish(ms_offset)
+        return builder
+
     def __eq__(self, other):
         return isinstance(other, self.__class__) and self.__dict__ == other.__dict__
 
