@@ -4,9 +4,17 @@
 # 3) we can import it into your module module
 # https://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
 
-__version__ = '1.51.3'
+__version__ = '1.53.2'
 
 release_notes = {
+    '1.53.2': """
+    Making it possible to maintain the map list in the open source portion of RLBot,
+    and adding support for NeonFields.
+    """,
+    '1.52.0': """
+    Adding a socket management utility in Python to help people connect directly to
+    RLBot.exe and write interesting scripts.
+    """,
     '1.51.3': """
     Adding support for standalone python bots, giving faster startup and more
     intuitive development workflow. Also brings support for per-bot virtual environments.
